'use strict';

/**
 * Module dependencies.
 */
var should = require('should'),
	mongoose = require('mongoose'),
	User = mongoose.model('User'),
	ShinyApp = mongoose.model('ShinyApp');

/**
 * Globals
 */
var user, shinyApp;

/**
 * Unit tests
 */
describe('Shiny app Model Unit Tests:', function() {
	beforeEach(function(done) {
		user = new User({
			firstName: 'Full',
			lastName: 'Name',
			displayName: 'Full Name',
			email: 'test@test.com',
			username: 'username',
			password: 'password'
		});

		user.save(function() {
			shinyApp = new ShinyApp({
				name: 'Shiny app Name',
				user: user,
				urlSuffix: 'testUrl'
			});

			done();
		});
	});

	describe('Method Save', function() {
		it('should be able to save without problems', function(done) {
			return shinyApp.save(function(err) {
				should.not.exist(err);
				done();
			});
		});

		it('should be able to show an error when try to save without name', function(done) {
			shinyApp.name = '';

			return shinyApp.save(function(err) {
				should.exist(err);
				done();
			});
		});
	});

	afterEach(function(done) {
		ShinyApp.remove().exec();
		User.remove().exec();

		done();
	});
});
